<?php

namespace Drupal\group_member_mgmt\GroupContentEnabler;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\group\Entity\GroupContentInterface;
use Drupal\group\Entity\GroupInterface;
use Drupal\group\Plugin\GroupContentEnabler\GroupMembership as GroupMembershipBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Extends GroupMembership GroupContentEnabler plugin.
 */
class GroupMembership extends GroupMembershipBase implements ContainerFactoryPluginInterface {

  /**
   * The group membership access handler service.
   *
   * @var \Drupal\group_member_mgmt\GroupMemberAccessHandler
   */
  protected $groupMemberAccessHandler;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $plugin = new static($configuration, $plugin_id, $plugin_definition);
    $plugin->groupMemberAccessHandler = $container->get('group_member_mgmt.member_access');
    return $plugin;
  }

  /**
   * {@inheritdoc}
   */
  public function getTargetEntityPermissions() {
    return $this->groupMemberAccessHandler
      ->getGroupMembershipPermissions($this->getGroupType(), parent::getTargetEntityPermissions());
  }

  /**
   * {@inheritdoc}
   */
  public function createAccess(GroupInterface $group, AccountInterface $account) {
    // Add logic, return if granted.
    if ($this->groupMemberAccessHandler->createMemberAccess($group, $account)) {
      return AccessResult::allowed();
    }

    // Default to standard access check.
    return parent::createAccess($group, $account);
  }

  /**
   * {@inheritdoc}
   */
  public function createEntityAccess(GroupInterface $group, AccountInterface $account) {
    if (!$this->definesEntityAccess()) {
      return AccessResult::neutral();
    }

    // Add logic, return if granted.
    if ($this->groupMemberAccessHandler->createMemberAccess($group, $account)) {
      return AccessResult::allowed();
    }

    // Default to standard access check.
    return parent::createEntityAccess($group, $account);
  }

  /**
   * {@inheritdoc}
   */
  protected function updateAccess(GroupContentInterface $group_content, AccountInterface $account) {
    // Add logic, return if granted.
    if ($this->groupMemberAccessHandler->updateMemberAccess($group_content, $account)) {
      return AccessResult::allowed();
    }

    // Default to standard access check.
    return parent::updateAccess($group_content, $account);
  }

  /**
   * {@inheritdoc}
   */
  protected function deleteAccess(GroupContentInterface $group_content, AccountInterface $account) {
    // Add logic, return if granted.
    if ($this->groupMemberAccessHandler->deleteMemberAccess($group_content, $account)) {
      return AccessResult::allowed();
    }

    // Default to standard access check.
    return parent::deleteAccess($group_content, $account);
  }

}
